package de.unisaar.jfap.ex2;

import static org.assertj.core.api.Assertions.assertThat;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

class MovieListSerializationTest {

  List<Movie> movieList;

  @BeforeEach
  void init() {
    movieList = new ArrayList<>();

    Movie movie1 = new Movie("Star Wars: Episode IV - A New Hope", 1977);
    movie1.addActor(new Actor("Mark", "Hamill"));
    movie1.addActor(new Actor("Harrison", "Ford"));
    movie1.addActor(new Actor("Carrie", "Fisher"));
    movieList.add(movie1);

    Movie movie2 = new Movie("Avengers: Endgame", 2019);
    movie2.addActor(new Actor("Robert", "Downey Jr."));
    movie2.addActor(new Actor("Chris", "Evans"));
    movie2.addActor(new Actor("Mark", "Ruffalo"));
    movieList.add(movie2);

    Movie movie3 = new Movie("The Lord of the Rings: The Return of the King", 2003);
    movie3.addActor(new Actor("Elijah", "Wood"));
    movie3.addActor(new Actor("Viggo", "Mortensen"));
    movie3.addActor(new Actor("Ian", "McKellen"));
    movieList.add(movie3);
  }

  @Test
  void testJavaSerialization() throws IOException, ClassNotFoundException {

    Path movieListPath = Paths.get("movieList.obj");

    // serialize
    try (ObjectOutputStream out = new ObjectOutputStream(Files.newOutputStream(movieListPath))) {
      out.writeObject(movieList);
    }
    // deserialize
    try (ObjectInputStream in = new ObjectInputStream(Files.newInputStream(movieListPath))) {
      @SuppressWarnings("unchecked")
      List<Movie> deserializedMovieList = (List<Movie>) in.readObject();
      assertThat(deserializedMovieList).isEqualTo(movieList);
      // equals only checks id, so manually check other fields
      for (int i = 0; i < deserializedMovieList.size(); i++) {
        compareMovies(deserializedMovieList.get(i), movieList.get(i));
      }
    }
  }

  @Test
  void testJsonSerialization() throws JsonProcessingException {

    ObjectMapper objectMapper = new ObjectMapper();

    // serialize
    String jsonString = objectMapper.writerWithDefaultPrettyPrinter().writeValueAsString(movieList);
    System.out.println(jsonString);

    // deserialize
    List<Movie> deserializedMovieList =
        objectMapper.readValue(jsonString, new TypeReference<List<Movie>>() {});
    assertThat(deserializedMovieList).isEqualTo(movieList);
    // equals only checks id, so manually check other fields
    for (int i = 0; i < deserializedMovieList.size(); i++) {
      compareMovies(deserializedMovieList.get(i), movieList.get(i));
    }
  }

  private void compareMovies(Movie movie1, Movie movie2) {
    assertThat(movie1.getTitle()).isEqualTo(movie2.getTitle());
    assertThat(movie1.getYear()).isEqualTo(movie2.getYear());
    assertThat(movie1.getActors()).isEqualTo(movie2.getActors());
  }
}
